# include	<math.h>
# include	"Utility.h"

# include	"DE.h"

namespace	ColorimetrySpace
{

DE::DE(float thisCIE76, float thisCIE94, float thisCMC, float thisCIEDE00)
	noexcept: CIE76(thisCIE76), CIE94(thisCIE94), CMC(thisCMC),
	CIEDE00(thisCIEDE00)
	{
	}

DE::DE(float LstarStandard, float astarStandard, float bstarStandard,
	float habStandard, float CabstarStandard, float LstarBatch,
	float astarBatch, float bstarBatch, float CabstarBatch, float CMCl,
	float CMCc, float CIE94kL, float CIE94kC, float CIE94kH,
	float CIEDE00kL, float CIEDE00kC, float CIEDE00kH) noexcept:
	CIE76(sqrt(Square(LstarStandard - LstarBatch) +
		Square(astarStandard - astarBatch) +
		Square(bstarBatch - bstarStandard)))
	{
	float	DL2;
	float	DC2;
	float	DH2;
	float	EFFE;
	float	Sc;
	float	Sh;
	float	Sl;
	float	G;
	float	ph1;
	float	ph2;
	float	pC1;
	float	pC2;
	float	pLave;
	float	pCave;
	float	phave;
	float	RT;
	float	DH;
	float	Temp;

	DL2 = Square(LstarStandard - LstarBatch);
	DC2 = Square(CabstarStandard - CabstarBatch);
	DH2 = Square(CIE76) - DL2 - DC2;
	CIE94 = sqrt(DL2 / Square(CIE94kL) +
		DC2 / Square(CIE94kC * (1 + 0.045 * CabstarStandard)) +
		DH2 / Square(CIE94kH * (1 + 0.015 * CabstarStandard)));
	Sc = 0.638 + 0.0638 * CabstarStandard / (1 + 0.0131 * CabstarStandard);
	EFFE = sqrt(Square(Square(CabstarStandard)) /
		(Square(Square(CabstarStandard)) + 1900));
	if (habStandard < 164 or habStandard > 345)
		Temp = 0.36 +
			fabs(0.4 * cos(DegreesToRadians(habStandard + 35)));
	else
		Temp = 0.56 +
			fabs(0.2 * cos(DegreesToRadians(habStandard + 168)));
	Sh = Sc * (Temp * EFFE + 1 - EFFE);
	if (LstarStandard < 16)
		Sl = 0.511;
	else
		Sl = 0.040975 * LstarStandard / (1 + 0.01765 * LstarStandard);
	CMC = sqrt(DL2 / Square(CMCl * Sl) + DC2 / Square(CMCc * Sc) +
		DH2 / (Sh * Sh));
	G = 0.5 * (1 - sqrt(1 / (1 +
		SeventhPower(25 / ((CabstarBatch + CabstarStandard) / 2)))));
	ph1 = Arctan(bstarBatch, Temp = astarBatch * (1 + G));
	pC1 = sqrt(Square(Temp) + Square(bstarBatch));
	Temp = astarStandard * (1 + G);
	pC2 = sqrt(Square(Temp) + Square(bstarStandard));
	ph2 = Arctan(bstarStandard, Temp);
	pLave = (LstarBatch + LstarStandard) / 2;
	pCave = (pC1 + pC2) / 2;
	phave = fabs(ph1 - ph2) < 180? (ph1 + ph2) / 2: (ph2 - 360 + ph1) / 2;
	Temp = 1 - 0.17 * cos(DegreesToRadians(phave - 30)) +
		0.24 * cos(DegreesToRadians(2 * phave)) +
		0.32 * cos(DegreesToRadians(3 * phave + 6)) -
		0.2 * cos(DegreesToRadians(4 * phave - 63));
	Sl = 1 + 0.015 * Square(pLave - 50) / sqrt(20 + Square(pLave - 50));
	Sc = 1 + 0.045 * pCave;
	Sh = 1 + 0.015 * pCave * Temp;
	Temp = 30 * exp(-Square((phave - 275) / 25));
	RT = -sin(DegreesToRadians(2 * Temp)) * 2 *
		sqrt(1 / (1 + SeventhPower(25 / pCave)));
	Temp = pC2 - pC1;
	DH = 2 * sqrt(pC1 * pC2) * sin(DegreesToRadians((ph2 - ph1)/ 2));
	CIEDE00 = sqrt(Square((LstarStandard - LstarBatch) / (CIEDE00kL * Sl)) +
		Square(Temp / (CIEDE00kC * Sc)) +
		Square(DH / (CIEDE00kH * Sh)) +
		RT * (Temp / (CIEDE00kC * Sc)) * (DH / (CIEDE00kH * Sh)));
	}

float	DE::GetCIE76(void) const noexcept
	{
	return CIE76;
	}

float	DE::GetCIE94(void) const noexcept
	{
	return CIE94;
	}

float	DE::GetCMC(void) const noexcept
	{
	return CMC;
	}

float	DE::GetCIEDE00(void) const noexcept
	{
	return CIEDE00;
	}

}
