//
//  main.cpp
//  ColorimetryTests
//
//  Created by Gabriele Simone on 22/11/20.
//

#include <iostream>
#include "main.h"

using namespace ColorimetrySpace;

void    TestColorDifferenceFormulae()
{
    float   LstarStandard, astarStandard, bstarStandard, habStandard, CabstarStandard,
            LstarBatch, astarBatch, bstarBatch, CabstarBatch,
            CMCl, CMCc,
            CIE94kL, CIE94kC, CIE94kH,
            CIEDE00kL, CIEDE00kC, CIEDE00kH;
    
    std::cout << "Test 4: Color difference formulae\n";
    LstarStandard = 82.7152;
    astarStandard = -4.2869;
    bstarStandard = 83.5603;
    habStandard = 92.9369;
    CabstarStandard = 83.6702;
    LstarBatch = 83.8845;
    astarBatch = -4.4633;
    bstarBatch = 72.9627;
    CabstarBatch = 73.0991;
    CMCl = 1.0;
    CMCc = 1.0;
    CIE94kL = 1.0;
    CIE94kC = 1.0;
    CIE94kH = 1.0;
    CIEDE00kL = 1.0;
    CIEDE00kC = 1.0;
    CIEDE00kH = 1.0;
    
    DE  MyColorDifference(LstarStandard, astarStandard, bstarStandard, habStandard, CabstarStandard,
        LstarBatch, astarBatch, bstarBatch, CabstarBatch,
        CMCl, CMCc, CIE94kL, CIE94kC, CIE94kH, CIEDE00kL, CIEDE00kC, CIEDE00kH);
    
    std::cout << "CIE 76: " << MyColorDifference.GetCIE76() << "\n";
    std::cout << "CMC: " << MyColorDifference.GetCMC() << "\n";
    std::cout << "CIE 94: " << MyColorDifference.GetCIE94() << "\n";
    std::cout << "CIEDE2000: " << MyColorDifference.GetCIEDE00() << "\n";
}

void    TestXYZtoCIE()
{
    XYZ     MyXYZ(39.8482, 60.00, 14.0038);
    XYZ     MyXYZn(95.04, 100.00, 108.88); // D65 Illuminant
    
    std::cout << "Test 3: From XYZ to CIE\n";
    xyY     MyxyY(MyXYZ);
    std::cout << "x: " << MyxyY.Getx() <<
        ", y: " << MyxyY.Gety() <<
        ", Y: " << MyxyY.GetYBig() << "\n";
    
    xyzcc   Myxyzcc(MyXYZ);
    std::cout << "x: " << Myxyzcc.GetX() <<
        ", y: " << Myxyzcc.GetY() <<
        ", z: " << Myxyzcc.GetZ() << "\n";
    
    Labstar     MyLabstar(MyXYZ, MyXYZn);
    std::cout << "L*: " << MyLabstar.GetLstar() <<
        ", a*: " << MyLabstar.Getastar() <<
        ", b*: " << MyLabstar.Getbstar() << "\n";
    
    LHabCabstar     MyLHabCabstar(MyLabstar);
    std::cout << "L*: " << MyLHabCabstar.GetLstar() <<
        ", hab: " << MyLHabCabstar.GetHab() <<
        ", Cab*: " << MyLHabCabstar.GetCabstar() << "\n";
    
    uvapostr    Myuvapostr(MyXYZ);
    std::cout << "u': " << Myuvapostr.Getuapostr() <<
        ", v': " << Myuvapostr.Getvapostr() << "\n";
    
    Luvstar     MyLuvstar(MyXYZ, MyXYZn);
    std::cout << "L*: " << MyLuvstar.GetLstar() <<
        ", u*: " << MyLuvstar.Getustar() <<
        ", v*: " << MyLuvstar.Getvstar() << "\n";
    
    LHuvCuvstar MyLHuvCuvstar(MyLuvstar);
    std::cout << "L*: " << MyLHuvCuvstar.GetLstar() <<
        ", huv: " << MyLHuvCuvstar.GetHuv() <<
        ", Cuv*: " << MyLHuvCuvstar.GetCuvstar() << "\n";
    
}

void    TestXYZtoFundamental()
{
    XYZ     MyXYZ(30.00, 20.00, 30.00);
 
    std::cout << "Test 2: From XYZ to Fundamental\n";
    LMS     MyLMS(MyXYZ, ColorRepresentationType::VOS);
    std::cout << "L: " << MyLMS.GetL() <<
        ", M: " << MyLMS.GetM() <<
        ", S: " << MyLMS.GetS() << "\n";
    
    lmscc   Mylmscc(MyLMS);
    std::cout << "l: " << Mylmscc.GetL() <<
        ", m: " << Mylmscc.GetM() <<
        ", s: " << Mylmscc.GetS() << "\n";
    
    lsY     MylsY(MyLMS);
    std::cout << "l: " << MylsY.Getl() <<
        ", s: " << MylsY.Gets() <<
        ", Y: " << MylsY.GetY() << "\n";
    
}

void    TestTristimulusfromIlluminantSpectra()
{
    Spectrum    ReferenceWhite(D65_1nm, sizeof(D65_1nm) / sizeof(SpectrumData),
                1.0, SpectrumType::Power);
    //Comment previous line and uncomment next line for Illuminant A
    /* Spectrum    ReferenceWhite(A_1nm, sizeof(A_1nm) / sizeof(SpectrumData),
                1.0, SpectrumType::Power); */
    
    unsigned int    Index;
    float   Xn31, Yn31, Zn31, Xn64, Yn64, Zn64, Pow;
    
    std::cout << "Test 1: Tristimulus from Illuminant Spectra\n";
    
    std::cout << "CMFCIE31.GetNumValues(): "
        << CMFCIE31.GetNumValues() << "\n";
    std::cout << "ReferenceWhite.GetNumValues(): "
        << ReferenceWhite.GetNumValues() << "\n";
    
    Xn31 = Yn31 = Zn31 = 0;
    for (Index = 0; Index < ReferenceWhite.GetNumValues(); ++Index)
    {
        Pow = ReferenceWhite.GetValue(Index);
        Xn31 += CMFCIE31.GetX(Index) * Pow;
        Yn31 += CMFCIE31.GetY(Index) * Pow;
        Zn31 += CMFCIE31.GetZ(Index) * Pow;
    }
    Xn31 = Xn31 / Yn31 * 100;
    Zn31 = Zn31 / Yn31 * 100;
    Yn31 = 100;
    
    std::cout << "CIE '31 Observer, X: " << Xn31 <<
        ", Y: " << Yn31 <<
        ", Z: " << Zn31 << "\n";
    
    Xn64 = Yn64 = Zn64 = 0;
    for (Index = 0; Index < ReferenceWhite.GetNumValues(); ++Index)
    {
        Pow = ReferenceWhite.GetValue(Index);
        Xn64 += CMFCIE64.GetX(Index) * Pow;
        Yn64 += CMFCIE64.GetY(Index) * Pow;
        Zn64 += CMFCIE64.GetZ(Index) * Pow;
    }
    Xn64 = Xn64 / Yn64 * 100;
    Zn64 = Zn64 / Yn64 * 100;
    Yn64 = 100;
    
    std::cout << "CIE '64 Observer, X_10: " << Xn64 <<
        ", Y_10: " << Yn64 <<
        ", Z_10: " << Zn64 << "\n";
}

int main(int argc, const char * argv[]) {
    std::cout << "Colorimetry Tests Start!\n";
    TestTristimulusfromIlluminantSpectra();
    TestXYZtoFundamental();
    TestXYZtoCIE();
    TestColorDifferenceFormulae();
    std::cout << "Colorimetry Tests End!\n";
    return 0;
}
